// Forlix SoBall
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2008-2011 Dominik Friedrichs


Player_SpawnBall(client)
{
  if(round_freeze_end < 0
  || GetTickedTime()-round_freeze_end < spawnprotection_time)
   return(WITHIN_SPAWNPROT_TIME);

  decl Float:eyepos[3];
  decl Float:eyeang[3];

  GetClientEyePosition(client, eyepos);
  GetClientEyeAngles(client, eyeang);

  decl Float:spapos[3];

  GetAngleVectors(eyeang, spapos, NULL_VECTOR, NULL_VECTOR);
  ScaleVector(spapos, float(spawndistance_max));
  AddVectors(eyepos, spapos, spapos);

  // free the path to the preferred spawnpoint,
  // doing a tracehull with the balls dimensions.
  // if this hits an obstacle, the ball will be
  // spawned closer to the player accordingly
  TR_TraceHullFilter(eyepos, spapos,
                     SOCC_MODEL_EXTENT_MIN,
                     SOCC_MODEL_EXTENT_MAX,
                     MASK_PLAYERSOLID,
                     TraceEntityFilterPlayer, client);

  if(!TR_DidHit())
  // trace did not hit, distance too great
    return(SPAWN_POINT_TOO_FAR);

  TR_GetEndPosition(spapos);

  new hitent = TR_GetEntityIndex();
  // get the entity that was hit, if any

  if(!allow_spawn_on_players
  && hitent > 0
  && hitent <= MaxClients)
  // spawnpoint lies on a player and thats disallowed
    return(SPAWN_POINT_ON_PLAYER);

  new ball = -1;

  if(GetEntityCount() >= GetMaxEntities()
  || (ball = CreateEntityByName(PHYS_CLASSNAME, -1)) == -1)
  // max amount of entities reached or spawn failed
    return(SPAWN_FAILED);

  SetEntityModel(ball, SOCC_MODELFILE);

  DispatchKeyValue(ball, "spawnflags", "1026");
  DispatchKeyValue(ball, "targetname", SOCC_CLASSNAME);

  TeleportEntity(ball, spapos, NULL_VECTOR, NULL_VECTOR);
  DispatchSpawn(ball);

  EmitAmbientSound(SPAWN_SOUND, spapos, ball, 80);

  // trigger the OnHealthChanged output so the entity gets its classname set.
  // setting that directly here using DispatchKeyValue somehow mixes up with
  // DispatchSpawn and makes the output hook not respond to some entities!
  SetVariantInt(64);
  AcceptEntityInput(ball, "SetHealth", client, ball);

  p_ballspawns[client]++;

  if(log_ball_spawns)
  {
    decl String:steamid[32];
    decl String:teamname[32];

    if(!GetClientAuthString(client, steamid, sizeof(steamid)))
      strcopy(steamid, sizeof(steamid), "STEAM_ID_PENDING");

    GetTeamName(GetClientTeam(client), teamname, sizeof(teamname));

    LogToGame(LOG_MSG_SPAWN_BALL, client, GetClientUserId(client),
    steamid, teamname, p_ballspawns[client], spawncount_per_round);
  }

  return(SPAWN_SUCCEEDED);
}


public bool:TraceEntityFilterPlayer(entity,
                                    contentsMask,
                                    any:client)
{
  if(entity != client
  || !entity)
    return(true);
  else
    return(false);
}


SetupEntities()
{
  // instantiate a game_score entity for score updates
  game_score_ref = EntIndexToEntRef(CreateEntityByName("game_score", -1));
  DispatchSpawn(game_score_ref);

  // rehook entity outputs
  UnhookEntityOutput(PHYS_CLASSNAME, "OnHealthChanged", Event_PropShot);
  HookEntityOutput(PHYS_CLASSNAME, "OnHealthChanged", Event_PropShot);

  return;
}
