// Forlix SoBall
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2008-2011 Dominik Friedrichs

static Handle:h_soball_menu = INVALID_HANDLE;


BuildSoBallMenu()
{
  h_soball_menu = CreateMenu(MenuHandler, MenuAction_Display|MenuAction_DisplayItem);
  SetMenuTitle(h_soball_menu, "MENU_TITLE_TEXT");
  AddMenuItem(h_soball_menu, "", "MENU_SPAWN_TEXT");
  AddMenuItem(h_soball_menu, "", "MENU_INFO_TEXT");
  SetMenuExitButton(h_soball_menu, true);

  return;
}


CancelSoBallMenu()
{
  CancelMenu(h_soball_menu);
  return;
}


public Action:SoccerMenu(client, args)
{
  if(!client
  || !IsClientInGame(client))
    return(Plugin_Continue);

  if(DeadTimeBlock(client))
    return(Plugin_Handled);

  // respond to plugin command
  if(IsPlayerAlive(client))
  {
    if(spawncount_per_round)
    {
      if(p_ballspawns[client] < spawncount_per_round)
        DisplayMenu(h_soball_menu, client, MENU_DISPLAYTIME);
      else
        PrintToChat(client, "%t", "SPAWN_LIMIT_REACHED_MSG");
    }
    else
      PrintToChat(client, "%t", "SPAWN_DISABLED_MSG");
  }
  else
    PrintToChat(client, "%t", "NO_SPAWN_WHEN_DEAD_MSG");

  return(Plugin_Handled);
}


public MenuHandler(Handle:menu,
                   MenuAction:action,
                   param1,
                   param2)
{
  decl String:phrase[64];
  decl String:transl[128];

  new client = param1;
  SetGlobalTransTarget(client);

  switch(action)
  {
    case MenuAction_Display:
    {
      new Handle:panel = Handle:param2;
      GetMenuTitle(menu, phrase, sizeof(phrase));

      Format(transl, sizeof(transl), "%t", phrase);
      SetPanelTitle(panel, transl);
    }

    case MenuAction_DisplayItem:
    {
      GetMenuItem(menu, param2, "", 0, _, phrase, sizeof(phrase));

      Format(transl, sizeof(transl), "%t", phrase);
      return(RedrawMenuItem(transl));
    }

    case MenuAction_Select:
    {
      if(param2 == 0
      && IsPlayerAlive(client))
      // player selected to spawn a ball
      {
        new result = Player_SpawnBall(client);

        if(result == SPAWN_SUCCEEDED)
          return(0);

        switch(result)
        // error occured, output the corresponding message
        {
          case WITHIN_SPAWNPROT_TIME:
            PrintHintText(client, "%t", "WITHIN_SPAWNPROT_TIME_MSG");

          case SPAWN_POINT_TOO_FAR:
            PrintHintText(client, "%t", "SPAWN_POINT_TOO_FAR_MSG");

          case SPAWN_POINT_ON_PLAYER:
            PrintHintText(client, "%t", "SPAWN_POINT_ON_PLAYER_MSG");

          case SPAWN_FAILED:
            PrintHintText(client, "%t", "SPAWN_FAILED_MSG");
        }

        EmitSoundToClient(client, SPAWN_FAILURE_SOUND);

        // redraw menu with rate-limit
        static Float:time_lmd[MAXPLAYERS+1];
        new Float:time_c = GetTickedTime();

        if(time_c >= time_lmd[client] + 0.3)
          DisplayMenu(h_soball_menu, client, MENU_DISPLAYTIME);

        time_lmd[client] = time_c;
      }
      else
      if(param2 == 1)
        // player selected to display help panel
        HelpPanel(client);
    }
  }

  return(0);
}


public HelpPanel(client)
{
  decl String:buf[256];

  new Handle:pn = CreatePanel();
  SetGlobalTransTarget(client);

  Format(buf, sizeof(buf), "Forlix SoBall v%s\n ", PLUGIN_VERSION);
  DrawPanelText(pn, buf);

  Format(buf, sizeof(buf), "%t\n ", "HELPPANEL_TEXT", PLUGIN_COMMAND);
  DrawPanelText(pn, buf);

  DrawPanelText(pn, "(c) 2008-2011 Forlix\nEmail: df@forlix.org\nWeb: http://forlix.org/\n ");

  Format(buf, sizeof(buf), "0. %t", "EXIT");
  DrawPanelText(pn, buf);

  SendPanelToClient(pn, client, HelpPanelClose, PANEL_DISPLAYTIME);
  CloseHandle(pn);

  return;
}


public HelpPanelClose(Handle:panel,
                      MenuAction:action,
                      param1,
                      param2)
{
  return;
}
